#ifndef _RL_H_
#define _RL_H_

// 定义强化学习的状态空间维度
#define STATE_DIM 4       // 状态空间的维度（简化）
#define ACTION_DIM 2      // 动作空间的维度
#define HIDDEN_DIM 16     // 隐藏层维度
#define REPLAY_BUFFER_SIZE 1000  // 经验回放缓冲区大小
#define EPSILON 10        // ε-greedy策略中的探索率（整数表示，10 表示 0.1）

// 定义强化学习的状态（使用整数）
struct rl_state {
    int vruntime;      // 进程的虚拟运行时间
    int tickets;       // 进程的彩票数量
    int cpu_load;      // CPU的负载（用总彩票数表示）
    int elapsed_time;  // 进程的已运行时间
};

// 定义强化学习的动作（使用整数）
struct rl_action {
    int delta_tickets;  // 彩票数量的调整值
    int next_pid;       // 下一个运行的进程ID
};

// 定义简化版的策略网络（使用整数权重）
struct simple_policy {
    int weights[STATE_DIM][ACTION_DIM];  // 策略网络的权重（整数）
};

// 定义复杂版的策略网络（使用整数权重）
struct complex_policy {
    int weights1[STATE_DIM][HIDDEN_DIM];  // 第一层权重
    int weights2[HIDDEN_DIM][ACTION_DIM]; // 第二层权重
    int bias1[HIDDEN_DIM];                // 第一层偏置
    int bias2[ACTION_DIM];                // 第二层偏置
};

// 定义经验回放缓冲区
struct experience {
    struct rl_state state;
    struct rl_action action;
    int reward;
    struct rl_state next_state;
};

struct replay_buffer {
    struct experience buffer[REPLAY_BUFFER_SIZE];
    int size;
    int head;
};

// 声明强化学习相关函数
void init_simple_policy(void);
void init_complex_policy(struct complex_policy *policy);
void init_replay_buffer(struct replay_buffer *buffer);
void update_target_policy(struct complex_policy *policy, struct complex_policy *target_policy);
struct rl_action choose_action(struct rl_state state);
struct rl_action choose_action_complex(struct rl_state state, struct complex_policy *policy);
struct rl_action epsilon_greedy(struct rl_state state, struct complex_policy *policy);
int calculate_reward(int vruntime, int tickets, int total_tickets);
void add_experience(struct replay_buffer *buffer, struct rl_state state, struct rl_action action, int reward, struct rl_state next_state);
struct experience sample_experience(struct replay_buffer *buffer);

// 新添加的函数声明（整数运算）
int calculate_target_value(int reward, struct rl_state next_state, struct complex_policy *policy);
int calculate_error(int reward, struct rl_state state, struct rl_state next_state, struct complex_policy *policy);
int predict_reward(struct complex_policy *policy, struct rl_state state);
void update_complex_policy(struct complex_policy *policy, int error);

#endif // _RL_H_